/*
 Canvas Draw - CD_XX Driver
*/

#include "cd.h"
#include "cdxx.h"
#include "cdprivat.h"
#include <stdlib.h> 
#include <stdio.h> 
#include <memory.h> 



typedef struct cdCanvasXX { 
  long bg, fg;          /* Cor de fundo e de frente */
  int w, h;             /* width e height do canvas em pixel */
  double w_mm, h_mm;    /* width e height do canvas em mm */ 
  int back_opacity;     /* Opacidade das primitivas */
  int write_mode;       /* REPLACE, XOR, NOT_XOR */
  int bpp;              /* numero de bits por pixel */
  int line_style;       /* Estilo das linhas */
  int line_width;       /* Espessura das linhas */
  int interior_style;   /* Estilo de preenchimento */
  int text_alignment;   /* Alinhamento do texto */
  int mark_type;        /* Tipo da marca */
  int mark_size;        /* Tamanho da marca */
  int poly_style;       /* Estilo do poligono */
  int hatch_style;      /* Estilo do hatch */

  struct 
    {
    int type_face;      /* COURIER, TIMES_ROMAN, HELVETICA              */
    int style;          /* PLAIN, ITALIC, BOLD, BOLD_ITALIC             */
    int size;           /* STANDARD, SMALL, LARGE                       */

    int max_width;     
    int height;        
    int ascent;        
    int descent;  
    } font;

  struct {
    int w,h;             /* largura e altura do stipple                 */
    int size;            /* tamanho maximo alocado                      */
    unsigned char *fgbg; /* indices de foreground(1) ou background (0)  */
  } stipple;

  struct {
    int w,h;            /* largura e altura do padrao de cores          */
    int size;           /* tamanho maximo alocado                       */
    long int *colors;   /* matriz de cores do CD                        */
  } pattern;

  struct {
    int on;             /* Indica se o clipping esta ativo */
    int xmin, ymin;         /* Define a regiao de clip */
    int xmax, ymax;         /* Define a regiao de clip */
  } clip;

  struct point {
    int x, y;      /* vertice do poligono */
  };

  struct {
    int n;               /* numero de pontos correntes */
    int size;            /* numero de pontos total */
    int mode;            /* OPEN_LINES, FILL, CLOSED_LINES, MARKS */
    struct point * pnt;  /* coordenadas do pixel no X */
  } poly;                /* armazenamento de pontos */
} cdCanvasXX;


static cdCanvasXX *xxActiveCanvas = NULL;


static void cdkillcanvas(cdCanvas *canvas)
{
  cdCanvasXX *cnv = (cdCanvasXX *)canvas->cnv;

  free(cnv->stipple.fgbg);
  free(cnv->pattern.colors);
  free(cnv->poly.pnt);
  free(cnv);
  free(canvas);
}

static int cdactivate(void *data)
{
  xxActiveCanvas = (cdCanvasXX *)data;
  return CD_OK;
}

static void cddeactivate(void)
{
  xxActiveCanvas = NULL;
}

static void cdflush(void)
{
}

static void cdclear(void)
{
}

static int cdplay(int xmin, int xmax, int ymin, int ymax, void *data)
{
  return CD_ERROR;
}

static int cdregistercallback(int cb, int(*func)(cdContext *driver, ...))
{
  return CD_ERROR;
}

static void cdgetcanvassize(int *width, int *height, double *mm_width, double *mm_height)
{
   if (width) *width  = xxActiveCanvas->w;
   if (height) *height = xxActiveCanvas->h;
   if (mm_width) *mm_width  = xxActiveCanvas->w_mm;
   if (mm_height) *mm_height = xxActiveCanvas->h_mm;
}

static int cdclip(int mode)
{
  int old_mode = xxActiveCanvas->clip.on;

  if (mode == CD_QUERY)
      return old_mode;

  xxActiveCanvas->clip.on = mode;
  return old_mode;
}

static void cdcliparea(int xmin, int xmax, int ymin, int ymax)
{
  xxActiveCanvas->clip.xmin = xmin;
  xxActiveCanvas->clip.ymin = ymin;
  xxActiveCanvas->clip.xmax = xmax;
  xxActiveCanvas->clip.ymax = ymax;

  if (xxActiveCanvas->clip.on == CD_CLIPON) 
	  cdclip(CD_CLIPON);
}

static void cdcanvas2raster(int *x, int *y)
{
}

static int cdgetcliparea(int *xmin, int *xmax, int *ymin, int *ymax)
{
  if (xmin) *xmin = xxActiveCanvas->clip.xmin;
  if (ymin) *ymin = xxActiveCanvas->clip.ymin;
  if (xmax) *xmax = xxActiveCanvas->clip.xmax;
  if (ymax) *ymax = xxActiveCanvas->clip.ymax;

  return xxActiveCanvas->clip.on;
}

static void cdline(int x1, int y1, int x2, int y2)
{
}

static void cdbox(int xmin, int xmax, int ymin, int ymax)
{
}

static void cdarc(int xc, int yc, int w, int h, double a1, double a2)
{
}

static void cdsector(int xc, int yc, int w, int h, double a1, double a2)
{
}

static void cdtext(int x, int y, char *s)
{
}

static void cdbegin(int mode)
{
  xxActiveCanvas->poly.mode = mode;
  xxActiveCanvas->poly.n = 0;
}

static void cdvertex(int x, int y)
{
  if (xxActiveCanvas->poly.n >= xxActiveCanvas->poly.size) { /* realoca array dos pontos */
    struct point *pnt;
    int newsize = (xxActiveCanvas->poly.size + 1) * 2;

    pnt = realloc(xxActiveCanvas->poly.pnt,newsize*sizeof(struct point));
    if (!pnt) 
       return;

    xxActiveCanvas->poly.pnt = pnt;
    xxActiveCanvas->poly.size = newsize;
  }

  xxActiveCanvas->poly.pnt[xxActiveCanvas->poly.n].x = x;
  xxActiveCanvas->poly.pnt[xxActiveCanvas->poly.n].y = y;
  xxActiveCanvas->poly.n++;
}

static void cdend(void)
{
  switch(xxActiveCanvas->poly.mode) 
  {
    case CD_CLOSED_LINES:
      cdVertex(xxActiveCanvas->poly.pnt[0].x, xxActiveCanvas->poly.pnt[0].y);
      /* continua */
    case CD_OPEN_LINES:
       break;
    case CD_FILL:
       cdVertex(xxActiveCanvas->poly.pnt[0].x, xxActiveCanvas->poly.pnt[0].y);
       break;
  }
}

static int cdbackopacity(int opacity)
{
  int old_bo = xxActiveCanvas->back_opacity;

  if (opacity == CD_QUERY)
      return old_bo;

  xxActiveCanvas->back_opacity = opacity;

  switch (opacity)
    {
    case CD_TRANSPARENT:
       break;
    case CD_OPAQUE:
       break;
    }

  return old_bo;
}

static int cdwritemode(int mode)
{
  int old_wm = xxActiveCanvas->write_mode;

  if (mode == CD_QUERY)
    return old_wm;

  xxActiveCanvas->write_mode = mode;

  switch (mode)
    {
    case CD_REPLACE:
       break;
    case CD_XOR:
       break;
    case CD_NOT_XOR:
       break;
    }

  return old_wm;
}

static int cdlinestyle(int style)
{
  int old_style = xxActiveCanvas->line_style;

  if (style == CD_QUERY)
     return old_style;

  switch (style)
  {
    case CD_CONTINUOUS :
      break;
    case CD_DASHED :
      break;
    case CD_DOTTED :
      break;
    case CD_DASH_DOT :
      break;
    case CD_DASH_DOT_DOT :
      break;
    default :
      return CD_ERROR;
  }

  xxActiveCanvas->line_style = style;
  return old_style;
}

static int cdlinewidth(int width)
{
  int old_width = xxActiveCanvas->line_width;

  if (width == CD_QUERY)
     return old_width;

  xxActiveCanvas->line_width = width;
  return old_width;
}

static int cdinteriorstyle (int style)
{
  int old_style = xxActiveCanvas->interior_style;

  if (style == CD_QUERY)
     return old_style;

  xxActiveCanvas->interior_style = style;

  switch (style)
  {
    case CD_SOLID:
      break;
    case CD_HATCH:
      cdHatch(xxActiveCanvas->hatch_style);
      break;
    case CD_STIPPLE:
      cdStipple(xxActiveCanvas->stipple.w, xxActiveCanvas->stipple.h, xxActiveCanvas->stipple.fgbg);
      break;
    case CD_PATTERN:
      cdPattern(xxActiveCanvas->pattern.w, xxActiveCanvas->pattern.h, xxActiveCanvas->pattern.colors);
      break;
    }

  return old_style;
}

static void cdhatch(int style)
{
   switch (style)
      {
      case CD_HORIZONTAL:
         break;
      case CD_VERTICAL:
         break;
      case CD_FDIAGONAL:
         break;
      case CD_BDIAGONAL:
         break;
      case CD_CROSS:
         break;
      case CD_DIAGCROSS:
         break;
      }
}

static void cdstipple(int w, int h, unsigned char *stipple)
{
  int t = w * h;
  int i;
  xxActiveCanvas->stipple.w = w;
  xxActiveCanvas->stipple.h = h;
  free(xxActiveCanvas->stipple.fgbg);
  xxActiveCanvas->stipple.fgbg = (unsigned char *) malloc(t);
  xxActiveCanvas->stipple.size = t;

  for (i = 0; i < t; i++)
    xxActiveCanvas->stipple.fgbg[i] = stipple[i];
}

static void cdpattern(int w, int h, long int *pattern)
{
  int t = w * h;
  int i;
  xxActiveCanvas->pattern.w = w;
  xxActiveCanvas->pattern.h = h;
  free(xxActiveCanvas->pattern.colors);
  xxActiveCanvas->pattern.colors = (long int *) malloc(t*sizeof(long int));
  xxActiveCanvas->pattern.size = t;

  for (i = 0; i < t; i++)
    xxActiveCanvas->pattern.colors[i] = pattern[i];
}

static void cdfont(int type_face, int style, int size)
{
 switch (type_face)
    {
    case CD_COURIER:
       break;
    case CD_TIMES_ROMAN:
       break;
    case CD_HELVETICA:
       break;
    case CD_SYSTEM:
       break;
    } 

  xxActiveCanvas->font.type_face = type_face;
  xxActiveCanvas->font.style = style;
  xxActiveCanvas->font.size = size;
  /* Falta atualizar ascent, descent, height e max_width */
}

static void cdnativefont(char *font)
{
}

static void cdfontdim(int *max_width, int *line_height, int *ascent, int *descent)
{
  if (max_width) *max_width = xxActiveCanvas->font.max_width;
  if (line_height) *line_height = xxActiveCanvas->font.height;
  if (ascent) *ascent = xxActiveCanvas->font.ascent;
  if (descent) *descent = xxActiveCanvas->font.descent;
}

static void cdtextsize(char *s, int *width, int *height)
{
}

static int cdtextalignment(int alignment)
{
  int old_alignment = xxActiveCanvas->text_alignment;

  if (xxActiveCanvas->text_alignment == CD_QUERY)
    return old_alignment;

  xxActiveCanvas->text_alignment = alignment;

  switch (xxActiveCanvas->text_alignment)
    {
    case CD_NORTH_EAST:
       break;
    case CD_EAST:
       break;
    case CD_BASE_RIGHT:
       break;
    case CD_SOUTH_EAST:
       break;
    case CD_NORTH:
       break;
    case CD_CENTER:
       break;
    case CD_BASE_CENTER:
       break;
    case CD_SOUTH:
       break;
    case CD_NORTH_WEST:
       break;
    case CD_WEST:
       break;
    case CD_BASE_LEFT:
       break;
    case CD_SOUTH_WEST:
       break;
    }

  return old_alignment;
}

static double cdtextorientation(double angle)
{
  return 0;
}

static int cdmarktype(int type)
{
  int old = xxActiveCanvas->mark_type;

  if (type != CD_QUERY)
    xxActiveCanvas->mark_type = type;

  return old;
}

static int cdmarksize(int size)
{
  int old = xxActiveCanvas->mark_size;

  if (size != CD_QUERY)
    xxActiveCanvas->mark_size = size;

  return old;
}

static int cdgetcolorplanes(void)
{
  return xxActiveCanvas->bpp;
}

static void cdpalette(int n, long int *palette, int mode)
{
  if (xxActiveCanvas->bpp > 8)
    return;
}

static long int cdbackground(long int color)
{
  long int old_bg = xxActiveCanvas->bg;
  if (color != CD_QUERY)
    xxActiveCanvas->bg = color;
  return old_bg;
}

static long int cdforeground(long int color)
{
  long int old_fg = xxActiveCanvas->fg;
  if (color != CD_QUERY)
    xxActiveCanvas->fg = color;
  return old_fg;
}

static void cdgetimagergb(unsigned char *r, unsigned char *g, unsigned char *b, int x, int y, int w, int h)
{
}

static void cdputimagergb(int iw, int ih, unsigned char *r, unsigned char *g, unsigned char *b, int x, int y, int w, int h)
{
}

static void cdputimagergba(int iw, int ih, unsigned char *r, unsigned char *g, unsigned char *b, unsigned char *a, int x, int y, int w, int h)
{
}

static void cdputimagemap(int iw, int ih, unsigned char *index, long int *colors, int x, int y, int w, int h)
{
}

static void cdpixel(int x, int y, long int color)
{
}

static void* cdcreateimage(int w, int h)
{
return NULL;
}

static void cdgetimage(void* image, int x, int y)
{
}

static void cdputimage(void* image, int x, int y)
{
}

static void cdputimagerect(void* image, int x, int y, int xmin, int xmax, int ymin, int ymax)
{
}

static void cdkillimage(void* image)
{
}

static void cdscrollimage(int xmin,int xmax, int ymin,int ymax, int dx,int dy)
{
}

static cdCanvas *cdcreatecanvas(void *data);

static cdFunctionTable cdXXFunctionTable =
{
  /* control */
  cdcreatecanvas,
  cdkillcanvas,
  cdactivate,
  cddeactivate,
  cdflush,
  cdclear,
  cdplay,
  cdregistercallback,

  /* coordinate transformation */
  cdgetcanvassize,
  cdclip,
  cdcliparea,
  cdcanvas2raster,
  cdgetcliparea,

  /* primitives */
  cdline,
  cdbox,
  cdarc,
  cdsector,
  cdtext,
  cdbegin,
  cdvertex,
  cdend,

  /* attributes */
  cdbackopacity,
  cdwritemode,
  cdlinestyle,
  cdlinewidth,
  cdinteriorstyle,
  cdhatch,
  cdstipple,
  cdpattern,
  cdfont,
  cdnativefont,
  cdfontdim,
  cdtextsize,
  cdtextalignment,
  cdtextorientation,
  cdmarktype,
  cdmarksize,

  /* color */
  cdgetcolorplanes,
  cdpalette,
  cdbackground,
  cdforeground,

  /* client images */
  cdgetimagergb,
  cdputimagergb,
  cdputimagergba,
  cdputimagemap,

  /* server images */
  cdpixel,
  cdcreateimage,
  cdgetimage,
  cdputimage,
  cdputimagerect,
  cdkillimage,
  cdscrollimage
};
 
static cdContext cdXX =
{
  &cdXXFunctionTable
};


cdContext* cdContextXX(void)
{
  return &cdXX;
}

static cdCanvas *cdcreatecanvas(void *data)
{
  char* filename = (char*)data;
  cdCanvasXX* new_xx;
  cdCanvas *canvas = (cdCanvas *)malloc(sizeof(cdCanvas));
  if (!canvas) 
	return NULL;

  new_xx = (cdCanvasXX *)malloc(sizeof(cdCanvasXX));
  if (!new_xx)
  {
    free(canvas);
    return NULL;
  }

  new_xx->fg = CD_BLACK;
  new_xx->bg = CD_WHITE;
  new_xx->back_opacity   = CD_TRANSPARENT;
  new_xx->write_mode     = CD_REPLACE;
  new_xx->line_style     = CD_CONTINUOUS;
  new_xx->line_width     = 0;
  new_xx->interior_style = CD_SOLID;
  new_xx->hatch_style    = CD_HORIZONTAL;

  new_xx->stipple.w = 32;
  new_xx->stipple.h = 32;
  new_xx->stipple.size = 32*32;
  new_xx->stipple.fgbg = (unsigned char *) malloc(new_xx->stipple.size*sizeof(unsigned char));
  memset(new_xx->stipple.fgbg, 0, new_xx->stipple.size);
  if (!new_xx->stipple.fgbg) 
     { 
     new_xx->stipple.w = 0; 
     new_xx->stipple.h=0; 
     }

  new_xx->pattern.w = 32;
  new_xx->pattern.h = 32;
  new_xx->pattern.size = 32*32;
  new_xx->pattern.colors = (long int *) malloc(new_xx->pattern.size*sizeof(long int));
  memset(new_xx->pattern.colors, 0, new_xx->pattern.size*sizeof(long int));
  if (!new_xx->pattern.colors) 
     {
     new_xx->pattern.w = 0; 
     new_xx->pattern.h = 0; 
     } 

  new_xx->poly.n = 0;
  new_xx->poly.size = 50;
  new_xx->poly.pnt = (struct point*)malloc(new_xx->poly.size*sizeof(struct point));
  if (!new_xx->poly.pnt) 
     new_xx->poly.size = 0;

  new_xx->text_alignment = CD_BASE_LEFT;
  new_xx->font.type_face = CD_COURIER;
  new_xx->font.style     = CD_PLAIN;
  new_xx->font.size      = CD_SMALL;
                                            
  new_xx->mark_type = CD_STAR;
  new_xx->mark_size = 10;

  new_xx->clip.on   = CD_CLIPOFF;
  new_xx->clip.xmin = 0;
  new_xx->clip.ymin = 0;
  new_xx->clip.xmax = new_xx->w - 1;
  new_xx->clip.ymax = new_xx->h - 1;

  canvas->drv = &cdXXFunctionTable;
  canvas->cnv = new_xx; 

  return canvas;
}

