/*
* global.h
*
* Carlos Henrique Levy
* 03 Sep 92
*/

#ifndef __IUPGLOBALH__
#define __IUPGLOBALH__

#include <stdlib.h>      /* malloc, realloc, free, NULL */
#include <string.h>      /* strdup */
#include <ctype.h>       /* toloer */

#ifndef _IUPCLASS_TYPE
#define _IUPCLASS_TYPE
typedef struct _Iclass Iclass;
#endif /* _IUPCLASS_TYPE */

int   iupstricmp (const char* s1, const char* s2);

#ifdef NO_STRDUP
char* strdup( const char* );
#endif

#define streq(a,b)      ((a == b) ? 0 : (*a == *b) ? strcmp(a,b) : 1)
#define strneq(a,b,n)      ((a == b) ? 0 : (*a == *b) ? strncmp(a,b,n) : 1)
#define strieq(a,b)     ((a == b) ? 0 : (tolower(*a) == tolower(*b)) ? iupstricmp(a,b) : 1)

#ifndef ABS
#define ABS(a) ((a)>0?(a):-(a))
#endif

#ifndef MIN
#define MIN(a,b) ((a)<(b)?(a):(b))
#endif

#ifndef MAX
#define MAX(a,b) ((a)>(b)?(a):(b))
#endif

#define FRAME_MARGEM_X 2
#define FRAME_MARGEM_Y 2	/* xview 12 */	/* windows =  18 */

#define REALLOC(p,s)	((p)==NULL?malloc(s):realloc(p,s))

#define STRBLOCK	20
#define SPACE	        32

#define NOSTR   "NO"
#define YESSTR  "YES"
#define NO      0
#define YES     1
#define NOTDEF -1

/*
* lex.h
*/
/* TOKENS */
#define TK_END          -1
#define TK_BEGP         1
#define TK_ENDP         2
#define TK_ATTR         3
#define TK_STR          4
#define TK_NAME         5
#define TK_NUMB         6
#define TK_SET          7
#define TK_COMMA        8
#define TK_FUNC         9
#define TK_ENDATTR     10

/* ERRORS */
#define FILENOTOPENED   1
#define NOTMATCH        2
#define NOTENDATTR      3
#define INVALID_NUMBER  4
#define PARSEERROR      5

int     iupLexInit       (char *filename);
void    iupLexClose      (void);
int     iupLexLookAhead  (void);
int     iupLexAdvance    (void);
int     iupLexFollowedBy (int t);
int     iupLexMatch      (int t);
int     iupLexSeenMatch  (int t, int *erro);
int     iupLexInt        (void);
char*   iupLexGetName    (void);
char*   iupLexName       (void);
float   iupLexGetNumber  (void);
int     iupLexGetFunc    (void);
int     iupLexError      (int n, ...);


/*
* symb.h
*/
typedef struct Isymb_
	{
	 char *nm;		/* name */
	 void *info;		/* info */
         int  aloc;
	 struct Isymb_ *n;	/* next */
	} Isymb;

typedef struct Ienv_
	{
	 Isymb **table;		/* array de ponteiros para Isymb,para a hash table */
	 int   size;		/* tamanho do array */
	} Ienv;


char *iupSymbGetName (Ienv *l, void *n);
char* iupSymbGetNth  (Ienv *l, int n);
void* iupSymbGet     (Ienv* l, char* n);
Ienv* iupSymbAdd     (Ienv* l, char* n, void* t);
Ienv* iupSymbAddDup  (Ienv* l, char* n, void* t);
void* iupSymbDel     (Ienv *l, char *n);
int   iupSymbHash    (char *key, int tab_size);
Ienv* iupSymbAlloc   (Ienv *l, int size);
void  IupFreeAllSymb (Ienv* l, void (*func)(char*, void*));

/*
* tree.h
*/
typedef enum
	{
	UNKNOWN_,  /* 0 */
	COLOR_,
	IMAGE_,
	BUTTON_,
	CANVAS_,
	DIALOG_,   /* 5 */
	FILL_,
	FRAME_,
	HBOX_,
	ITEM_,
	SEPARATOR_,/* 10 */
	SUBMENU_,
	KEYACTION_,
	LABEL_,
	LIST_,
	MENU_,     /* 15 */
	RADIO_,
	TEXT_,
	TOGGLE_,
	VBOX_,     
	ZBOX_,
	MULTILINE_,
	USER_,    
        CONTROL_
	} Ielemtype;

typedef struct Ihandle_	Ihandle;

typedef int (*Iclosecb)   (Ihandle *);
typedef int (*Icallback)  (Ihandle *, ...);
typedef int (*Itogglecb)  (Ihandle *, int);
typedef int (*Ibuttoncb)  (Ihandle *);
typedef int (*Ieditcb)    (Ihandle *, int);
typedef int (*Itextcb)    (Ihandle *, int, char *);
typedef int (*Ilistcb)    (Ihandle *, char *, int, int);

typedef int (*Imatrixcb)  (Ihandle *, int, int, int, int);
typedef int (*Iitemcb)    (Ihandle *, int, int);
typedef int (*Ieditioncb) (Ihandle *, int, int, int);

typedef int (*Iscrollcb)  (Ihandle *, int, double, double);

typedef int (*Imousemvcb) (Ihandle*, int, int, char *);
typedef int (*Imousecb)   (Ihandle*, int, int, int, int, char *);
typedef int (*Iresizecb)  (Ihandle*, int, int);
typedef int (*Irepaintcb) (Ihandle*, double, double);
typedef int (*Idropfilescb) (Ihandle*, char *, int, int, int);

typedef struct Ikeyact_   Ikeyact;
typedef struct Ilist_     Ilist;

typedef struct Isize_     Isize;
typedef union  Iattrib_   Iattrib;

struct Tframe_
	{
	 int bx;
	 int by;
	};

struct Ikeyact_
	{
	 Ienv* list;
	};

struct Ilist_
	{
         void* data;
         void* dddata;
	};

struct Isize_
	{
	 int width;
	 int height;
	};

union Iattrib_
	{
         void *data;
	 Ikeyact k;
	 Ilist lst;
	};

struct Ihandle_
	{
         int ident;		/* serial number */
	 Ielemtype iet;		/* type of interface element */
	 Ienv* enviroment;
         Iclass* iclass;
	 void* handle;
	 int haveusersize;
         int x,y;               /* upper-left corner */
	 Isize natural;
	 Isize current;
	 Iattrib attrib;
	 Ihandle* next;         /* brother */
	 Ihandle* firstchild;
	 Ihandle* parent;
	};

#include "macros.h"


Ihandle* iupTreeCreateNode (Ihandle* root);
Ihandle* iupTreeCopyNode   (Ihandle* root, Ihandle *n);
Ihandle* iupTreeAppendNode (Ihandle* root, Ihandle* n);
Ihandle* iupTreeFindNode   (int n);
void     iupTreePrint      (Ihandle* root);

/*
* driver
*/
int      iupButtonSize     (Ihandle* e, char *s, int *w, int *h);
int      iupCanvasSize     (Ihandle* e, int *w, int *h);
int      iupBoxSize        (Ihandle* e, int *w, int *h);
int      iupFillSize       (Ihandle* e, int *w, int *h);
int      iupFrameSize      (Ihandle* e, int *w, int *h);
int      iupHboxSize       (Ihandle* e, int *w, int *h);
int      iupLabelSize      (Ihandle* e, char *s, int *w, int *h);
int      iupMultiLineSize  (Ihandle* e, int *w, int *h);
int      iupListSize       (Ihandle* e, int *w, int *h);
int      iupRadioSize      (Ihandle* e, char *s, int *w, int *h);
int      iupTextSize       (Ihandle* e, int *w, int *h);
int      iupToggleSize     (Ihandle* e, char *s, int *w, int *h);
int      iupVboxSize       (Ihandle* e, int *w, int *h);
int      iupZboxSize       (Ihandle* e, int *w, int *h);
void     iupUpdateDrv      (Ihandle* n, char *a, char* v, char *ov);
char*    iupGetValue       (Ihandle* n);
char*    iupGetVisible     (Ihandle* n);
void     iupGetCharSize    (Ihandle* e, int *w, int *h);
void     iupGetTextSize(Ihandle* h, char* text, int* size);
void     iupDestroyHandle  (Ihandle* n);
void     iupdrvCreateNativeObject (Ihandle *n);
void     iupdrvCreateObject(Ihandle* self, Ihandle* parent );
void     iupdrvCreateObjects(Ihandle *n);
void     iupdrvResizeObjects(Ihandle *n);
void     iupdrvUpdateAttrs (Ihandle* n);
void     iupdrvSetIdleFunction (Icallback f);
char*    iupdrvGetDefault  (Ihandle* n, char *a);
char*    iupdrvGetAttribute(Ihandle* n, char *a);
void     iupdrvSetAttribute(Ihandle* n, char *a, char *v);
int      iupdrvWindowSizeX (Ihandle* n, int scale);
int      iupdrvWindowSizeY (Ihandle* n, int scale);
int      iupdrvStoreAttribute(Ihandle* n, char* attr);
int      iupdrvPopupMenu(Ihandle *n, int x, int y);
int      iupdrvPopupDialog(Ihandle *n, int x, int y);

Iclass  *iupLexGetClass   (void);
void     iupdrvDetachMenuBar (Ihandle *h);

/*
* env.h
*/
char *iupstrregister(char *str);
int   iupCheck     (Ihandle *n, char *a);
void  iupSetEnv    (Ihandle* n, char* e, char* v);
void  iupStoreEnv  (Ihandle* n, char* e, char* v);
char* iupGetEnv    (Ihandle* n, char* e);
int   iupGetWidth  (Ienv* e);
int   iupGetHeight (Ienv* e);
int   iupGetColor  (Ihandle* n, char *a, unsigned int *r, unsigned int *g, unsigned int *b);
int   iupGetRGB (char *color, unsigned int *r, unsigned int *g, unsigned int *b);
int   iupGetSize   (Ihandle *e, int* w, int *h);
int   iupGetFillSize(Ihandle *e, int* w, int *h);
char* iupGetTitle  (Ihandle *n);
void  iupCreateGlobalEnv(void);
void  iupDestroyGlobalEnv(void);

/*
* make.h
*/
Ihandle* iupMkHbox      (Ihandle* box);
Ihandle* iupMkMenu      (Ihandle* menu);
Ihandle* iupMkVbox      (Ihandle* box);
Ihandle* iupMkZbox      (Ihandle* box);
Ihandle* iupMkKeyAction (Ienv* list);

/*
* calcsize.h
*/

int  iupSetSize        (Ihandle *root);
int  iupReSize         (Ihandle *root, int w, int h);
void iupSetNaturalSize (Ihandle *n);
void iupSetCurrentSize (Ihandle *n, int w, int h);

/*
* dialogs.h
*/
void     iupDLpush   (Ihandle *n);
Ihandle* iupDLpop    (void);
void     iupDLdelete (Ihandle *n);
Ihandle* iupDLFirst  (void);
Ihandle* iupDLNext   (void);
#define  foralldialogs(_)       for((_)=iupDLFirst();(_);(_)=iupDLNext())

/*
* key.h
*/
char *iupEncodeKey(int c);
int iupDecodeKey (char *c);

/* nfield.c */
int iupAcceptFocus    (Ihandle *h);

/* iupcpi */

void      IupKill (Ihandle *root);
void      IupRegisterLED (char *name, Iclass *ic);


#include "iup.h"

#endif
