#ifndef IUPCPI_H
#define IUPCPI_H

#include <ctype.h>

/* Clientes nao tem acesso `a estrutura interna
   da Iclass */
 
#ifdef __cplusplus
extern "C" {
#endif

#ifndef _IUPCLASS_TYPE
#define _IUPCLASS_TYPE
typedef struct _Iclass Iclass;
#endif /* _IUPCLASS_TYPE */

#ifndef __IUPGLOBALH__
#define NO      0
#define YES     1
#define NOTDEF -1

#ifndef MIN
#define MIN(a,b) ((a)<(b)?(a):(b))
#endif

#ifndef MAX
#define MAX(a,b) ((a)>(b)?(a):(b))
#endif

#ifndef streq
#define streq(a,b)      ((a == b) ? 0 : (*a == *b) ? strcmp(a,b) : 1)
#endif

#ifndef strieq
int     iupstricmp (const char* s1, const char* s2);
#define strieq(a,b)     ((a == b) ? 0 : (tolower(*a) == tolower(*b)) ? iupstricmp(a,b) : 1)
#endif


typedef int (*Ieditcb)    (Ihandle *, int);
typedef int (*Itextcb)    (Ihandle *, int, char *);
typedef int (*Iitemcb)    (Ihandle *, int, int);
typedef int (*Ieditioncb) (Ihandle *, int, int, int);
typedef int (*Imatrixcb)  (Ihandle *, int, int, int, int);
typedef int (*Ibuttoncb)  (Ihandle *);

/* prototipos de funcoes internas da IUP 
   acessiveis aos controles */

int   iupGetColor(Ihandle* n, char *attr, unsigned int *r, unsigned int *g, unsigned int *b);
int   iupGetRGB (char *color, unsigned int *r, unsigned int *g, unsigned int *b);
void  iupGetCharSize (Ihandle* n, int *w, int *h);
int   iupGetSize (Ihandle* e, int* w, int *h);
void  iupSetEnv(Ihandle* n, char* e, char* v);
void  iupStoreEnv(Ihandle* n, char* e, char* v);
char* iupGetEnv(Ihandle* self, char* attr);
int   iupCheck (Ihandle *n, char *a);
void  iupdrvResizeObjects (Ihandle *n);
void  iupGetTextSize(Ihandle* h, char* text, int* size);
void  IupKill (Ihandle *root);
Ihandle* IupPreviousField (Ihandle *h);
Ihandle* IupNextField (Ihandle *h);
char* iupMapFont(char* iup_font);

/* calcsize.c */
int iupSetSize(Ihandle* root);

/* tree.c */
Ihandle* iupTreeAppendNode(Ihandle* root, Ihandle *n);

/* lex.c */
void IupRegisterLED (char *name, Iclass *ic);

#endif /* __IUPGLOBALH__ */

typedef int (*Ifindchild_fn) ( Ihandle*,Ihandle*);

/* Funcoes de acesso `a estrutura de um elemento Iup (Ihandle) */

void* iupgetdata(Ihandle* self);
void  iupsetdata(Ihandle* self,void* data);
char* iupGetSymbol(Ihandle* self, char* n);
void  iupAddSymbol(Ihandle* self, char* attr, char* value);
void  iupSetPosX(Ihandle* self, int x);
void  iupSetPosY(Ihandle* self, int y);
int   iupGetPosX(Ihandle* self);
int   iupGetPosY(Ihandle* self);
void  iupSetCurrentWidth(Ihandle* self, int w);
void  iupSetCurrentHeight(Ihandle* self, int h);
int   iupGetCurrentWidth(Ihandle* self);
int   iupGetCurrentHeight(Ihandle* self);
void* iupGetNativeHandle(Ihandle* self);
void  iupSetNativeHandle(Ihandle* self, void* handle);
void  iupSetNaturalWidth(Ihandle* self, int w);
void  iupSetNaturalHeight(Ihandle* self, int h);
int   iupGetNaturalWidth(Ihandle* self);
int   iupGetNaturalHeight(Ihandle* self);
int   iupHorResizable(Ihandle* self);
int   iupVertResizable(Ihandle* self);
Ihandle* iupGetParent( Ihandle* self );
Ihandle* iupFindChild( Ihandle* self, Ifindchild_fn fn);
Ihandle* iupGetFirstChild(Ihandle* self);
Ihandle* iupGetBrother(Ihandle* self);


/* Funcoes de acesso `a Iclass  */

/*typedef void (*Imethod)(Ihandle *, ...);*/

typedef void *Imethod;


Iclass *iupCreateNewClass(char *name, char *format);

int iupSetClassMethod(Iclass *ic, char *method, Imethod func);
/*Imethod iupGetClassMethod(Iclass *ic, char *method);*/

/* Funcoes de acesso aos metodos CPI */

Ihandle* IupCreate( Iclass* iclass, ... );
Ihandle* IupCreateFromArray( Iclass* iclass, void** args );

int iupmethSetNaturalSize( Ihandle* self);
void  iupmethSetCurrentSize( Ihandle* self, int w, int h );
int   iupmethGetSize( Ihandle* self, int* w, int* h );
void  iupmethSetPosition ( Ihandle* self, int w, int h );
void  iupmethDestroy( Ihandle* self );
void  iupmethMap( Ihandle* self, Ihandle* parent );
void  iupmethUnmap( Ihandle* self );
void  iupmethSetAttribute( Ihandle* self, char* attr, char* value );
char* iupmethGetAttribute( Ihandle* self, char* attr);
char* iupmethGetDefaultAttr( Ihandle* self, char* attr);
char* iupmethGetClassName( Ihandle* self );


/* prototipos para os metodos default */

int   ctrsetnaturalsize (Ihandle* self);
void  ctrsetcurrentsize (Ihandle* self, int w, int h);
int   ctrgetsize (Ihandle* self, int* w, int* h);
int   ctrgetsizevar (Ihandle* self, int* w, int* h);
void  ctrsetposition (Ihandle* self, int w, int h);
Ihandle* ctrcreate (Iclass* self, void** array);
void  ctrdestroy (Ihandle* self);
void  ctrmap (Ihandle* self, Ihandle* parent);
void  ctrunmap (Ihandle* self);
void  ctrsetattr (Ihandle* self, char* attr, char* value);
char* ctrgetattr (Ihandle* self, char* attr);
char* ctrgetdefaultattr (Ihandle* self, char* attr);

#ifdef __cplusplus
}
#endif


/* Nomes dos metodos */
#define ICPI_SETNATURALSIZE "SETNATURALSIZE"
#define ICPI_SETCURRENTSIZE "SETCURRENTSIZE"
#define ICPI_GETSIZE "GETSIZE"
#define ICPI_SETPOSITION "SETPOSITION"
#define ICPI_CREATE "CREATE"
#define ICPI_DESTROY "DESTROY" 
#define ICPI_MAP "MAP"
#define ICPI_UNMAP "UNMAP"
#define ICPI_SETATTR "SETATTR"
#define ICPI_GETATTR "GETATTR"
#define ICPI_GETDRFAULT  "GETATTR"
#define ICPI_POPUP "POPUP"


#endif /* IUPCPI_H */

