/*==============================================================================
*NOME:   CALLBACKS.C
*
*  Projeto:  Editor 2D
*
*  Verso:              V 1.00
*  Autores :   Juliana Carpes Imperial        - 9714169-4
*              Marcelo Gattass
*
*Descrio do mdulo de declarao
*     Objetivo:
*        - Implementar as callbacks de interface.
*=============================================================================*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <cd.h>
#include "callbacks.h"
#include "interface.h"
#include "operacao.h"
#include "canvas.h"


/* Callback da acao da Opcao "Open" do menu "File" */
/* ----------------------------------------------- */
int open_cb( Ihandle* self ) {

   tpListaObjs* MeusObjs  ;  /* lista de objetos */   
   Ihandle*     IupCanvas ;  /* handle do IUP canvas */
   char *filename;
     
/* pega lista de objetos */
   MeusObjs = ( tpListaObjs* )IupGetAttribute( self, SZ_OBJETOS ) ; 
   
/* pega handle do IUP canvas */
   IupCanvas = ( Ihandle* )IupGetAttribute( self, "CanvasContext" ) ;
  
    /* recupera o nome do arquivo selecionado */
    filename = szMenu( IUP_OPEN ) ;
    if (!filename) return IUP_DEFAULT; /* operao cancelada */

    /* le os objetos do arquivo selecionado */
   if( objRead( filename, MeusObjs ) == -1 ) {
     IupMessage( SZ_CGI2000, SZ_ARQ_LET ) ;
     return IUP_DEFAULT;
   } /* if */

   printf( "\nOpen." ) ;

   /* redesenha o canvas */
   objDraw( IupCanvas, MeusObjs ) ;

    /* retorna o controle para o gerenciador de eventos (main loop) */
   return IUP_DEFAULT;
}	


/* Callback da acao da Opcao "Save" do menu "File" */
/* ----------------------------------------------- */
int save_cb( Ihandle* self ) {

/* pega nome de arquivo armazenado */
   char* szNomArq = szGuardarNomArq( 0 ) ;
 
/* pega lista de objetos */
   tpListaObjs* MeusObjs = ( tpListaObjs* )IupGetAttribute( self, SZ_OBJETOS ) ;     
   
/* se no h nome armazenado, deve-se escolher um nome */
   if ( strcmp( szNomArq, " " ) == 0 ) {

   /* recupera o nome do arquivo selecionado */
      szNomArq = szMenu( IUP_SAVE ) ;
      
      if ( szNomArq == 0 ) {
         return IUP_DEFAULT; /* operao cancelada */
      } /* if */
   
   } /* if */         
   
   /* grava os objetos no arquivo selecionado */
   if( objSave( szNomArq, MeusObjs ) == -1 ) {
      IupMessage( SZ_CGI2000, SZ_ARQ_ESC ) ;
      return IUP_DEFAULT ; /* no salvou */
   } /* if */

   printf( "\nSave." ) ;   /* salvou */

   return IUP_DEFAULT ;
} 


/* Callback da acao da Opcao "Exit" do menu "File" */
/* ----------------------------------------------- */
int exit_cb( Ihandle* self ) {

   Ihandle*     IupCanvas ; /* handle do canvas */
   tpListaObjs* MeusObjs  ; /* lista de objetos */

/* pega lista de objetos */
   MeusObjs = ( tpListaObjs* )IupGetAttribute( self, SZ_OBJETOS ) ;
   
/* pega handle do IUP canvas */
   IupCanvas = ( Ihandle* )IupGetAttribute( self, "CanvasContext" ) ;
 
   if ( MeusObjs != NULL ) {   /* desaloca lista se ela existir */
      free( MeusObjs ) ;
   } /* if */ 
   
   canvasDestroy( IupCanvas ) ; /* destroi canvas */

   IupMessage( SZ_CGI2000, SZ_TCHAU ) ;

  printf("\nExit");
   
  /* Retornando para o "iup main loop" com instrucao para ele encerrar */
  return IUP_CLOSE;
}


/* Callback da acao da Opcao "Info" do menu */
/* ----------------------------------------- */
int info_cb(void)
{
   IupMessage( SZ_CGI2000, SZ_JULIANA ) ;

  /* Retornando para o "iup main loop" */
  return IUP_DEFAULT;
}


/* Callback dos botoes do mouse    */
/* ----------------------------    */
/* handle - Handle IUP do canvas   */
/* button - IUP_BUTTON1 = Esquerdo */
/*          IUP_BUTTON2 = Meio     */
/*          IUP_BUTTON3 = Direito  */
/* pressed - 1 = pressionado, 0 = solto */
/*  (x, y) - Coordenadas do evento */
int button_cb(Ihandle* iupcanvas, int button, int pressed, int x,int y) {
    int b;

    /* Tratando o evento */

    switch (button) {
	case IUP_BUTTON1:
        b=1;
		break;
	case IUP_BUTTON2:
		b=2;
		break;
	case IUP_BUTTON3:
		b=3;
		break;
   
   default:
      b = 0 ;
      break ;

	}

    if (pressed) {
        printf("\nBotao %d pressionado em (%d, %d)", b, x, y);
    } 
    else {
        printf("\nBotao %d solto em (%d, %d)\n", b, x, y);
    }

	/* Retornando para o "iup main loop" */
	return IUP_DEFAULT;
}


/* Callback do movimento do mouse no canvas */
/* ---------------------------------------- */
/* handle - Handle IUP do canvas   */
/* (x, y) - Coordenadas do evento  */
int motion_cb(Ihandle* iupcanvas, int x, int y)
{

  /* Tratando o evento */
  printf("<%d,%d>",x,y);

  /* Retornando para o "iup main loop" */
  return IUP_DEFAULT;
}


/* Callback de resize do Canvas   */
/* ----------------------------   */
/* handle - Handle IUP do canvas  */
/* width - Nova largura do canvas */
/* height - Nova altura do canvas */
int resize_cb(Ihandle* iupcanvas, int width, int height)
{
   tpListaObjs* MeusObjs ; /* lista de objetos */
   
/* pega lista de objetos */
   MeusObjs = ( tpListaObjs* )IupGetAttribute( iupcanvas, SZ_OBJETOS ) ;

/* Tratando o evento */
   objDraw( iupcanvas, MeusObjs ) ;  /* redesenha o canvas */

  printf("\nResize:w=%d,h=%d",width,height);

  /* Retornando para o "iup main loop" */
  return IUP_DEFAULT;
}


/* Callback de repaint do Canvas */
/* ----------------------------- */
/* handle - Handle IUP do canvas */
int repaint_cb(Ihandle* iupcanvas)
{
   tpListaObjs* MeusObjs ; /* lista de objetos */
   
/* pega lista de objetos */
   MeusObjs = ( tpListaObjs* )IupGetAttribute( iupcanvas, SZ_OBJETOS ) ;

/* Tratando o evento */
   objDraw( iupcanvas, MeusObjs ) ;  /* redesenha o canvas */ 

  printf("\n Repaint");

  /* Retornando para o "iup main loop" */
  return IUP_DEFAULT;
}


/* Callback de mudanca no modo de selecao e manipulacao direta (Sel) */
/* ---------------------------------------------*/
/* handle - Handle IUP do toggle */
/* val = 1 se toggle foi para o estado on; e 0 se foi para o estado off */
int sel_cb(Ihandle* tog ,int val)
{

/* ao de boto agora registrada para selecionar objetos */
   IupSetFunction( "button_act", iSelBotaoCb ) ;
   
  printf("\nSelecao, modo %d",val);

  /* Retornando para o "iup main loop" */
  return IUP_DEFAULT;
}


/* Callback de mudanca no modo cria retangulos (Rect) */
/* ------------------------------------------------------*/
/* handle - Handle IUP do toggle */
/* val = 1 se toggle foi para o estado on; e 0 se foi para o estado off */
int rect_cb(Ihandle* tog ,int val)
{
/* ao de boto agora registrada para desenhar retngulo */
   IupSetFunction( "button_act", iRetBotaoCb ) ; 

  printf("\nRectangulo, modo %d",val);

  /* Retornando para o "iup main loop" */
  return IUP_DEFAULT;
}


/* Callback de mudanca no modo cria elipses (Ellipse) */
/* ------------------------------------------------------*/
/* handle - Handle IUP do toggle */
/* val = 1 se toggle foi para o estado on; e 0 se foi para o estado off */
int ellipse_cb(Ihandle* tog ,int val)
{
/* ao de boto agora registrada para desenhar elipse */
   IupSetFunction( "button_act", iEliBotaoCb ) ;

  printf("\nEllipse, modo %d",val);

  /* Retornando para o "iup main loop" */
  return IUP_DEFAULT;
}


/* Callback de mudanca no modo cria triangulo (Triang) */
/* ------------------------------------------------------*/
/* handle - Handle IUP do toggle */
/* val = 1 se toggle foi para o estado on; e 0 se foi para o estado off */
int triang_cb(Ihandle* tog ,int val)
{
/* ao de boto agora registrada para desenhar tringulo */
   IupSetFunction( "button_act", iTriBotaoCb ) ;

  printf("\nTriangulo, modo %d",val);

  /* Retornando para o "iup main loop" */
  return IUP_DEFAULT;
}


/*==============================================================================
*Nome: iExclusaoCb
*=============================================================================*/

int iExclusaoCb( Ihandle* Tog   ,
                 int      iValor ) {

/* ao de boto agora registrada para excluir objeto */
   IupSetFunction( "button_act", iExcBotaoCb ) ;

   printf( "\nExcluso, modo %d ", iValor ) ;
   return IUP_DEFAULT ;  /* retornando para o "iup main loop" */

} /* iExclusaoCb */


/*==============================================================================
*Nome: iNovoCb
*=============================================================================*/

int iNovoCb( Ihandle* self ) {
   
/* pega lista de objetos */
   tpListaObjs* MeusObjs  = ( tpListaObjs* )IupGetAttribute( self, 
                                                             SZ_OBJETOS ) ;
/* pega handle do Iup canvas */
   Ihandle*     IupCanvas = ( Ihandle* )IupGetAttribute( self, 
                                                         "CanvasContext" ) ;

   szGuardarNomArq( " " ) ;    /* inicializa o nome de arquivo */

/* desseleciona quem estiver selecionado */
   Desselecionar( -1, MeusObjs, IupCanvas ) ;
   
   canvasClear( IupCanvas ) ; /* apaga o que est no canvas */
   ZerarObjetos( MeusObjs ) ; /* zera o nmero de objetos */
   return IUP_DEFAULT ;       /* Retornando para o "iup main loop" */

} /* iNovoCb */


/*==============================================================================
*Nome: iSalvarComoCb
*=============================================================================*/

int iSalvarComoCb( Ihandle* self ) {

   tpListaObjs* MeusObjs ;  /* lista de objetos */
   char*        szNomArq ;  /* nome do arquivo 8/

/* pega lista de objetos */
   MeusObjs = ( tpListaObjs* )IupGetAttribute( self, SZ_OBJETOS ) ;

   szNomArq = szMenu( IUP_SAVE ) ; /* recupera o nome do arquivo selecionado */
   if ( szNomArq == 0 ) {
      return IUP_DEFAULT; /* operao cancelada */
   } /* if */

/* grava a lista de objetos no arquivo selecionado */
   if( objSave( szNomArq, MeusObjs ) == -1 ) {
      IupMessage( SZ_CGI2000, SZ_ARQ_ESC ) ;
      return IUP_DEFAULT ; /* no salvou */
   } /* if */

   printf( "\nSave." ) ;   /* salvou */

/* retorna o controle para o gerenciador de eventos (main loop) */
   return IUP_DEFAULT ;

} /* iSalvarComoCb */


/*==============================================================================
*Nome: iAjudaCb
*=============================================================================*/

int iAjudaCb( void ) {
   IupMessage( SZ_CGI2000, SZ_HELP ) ;
   return IUP_DEFAULT ; /* Retornando para o "iup main loop" */
} /* iAjudaCb */   


/*==============================================================================
*Nome: iSelBotaoCb
*=============================================================================*/

int iSelBotaoCb( Ihandle* IupCanvas , 
                 int      iBotao    , 
                 int      iPress    , 
                 int      iCoordX   ,
                 int      iCoordY         ) {

   tpListaObjs* MeusObjs ; /* lista de objetos */
   static int   iAchou   ; /* vai ser o ndice do objeto */  

/* pega lista de objetos */
   MeusObjs = ( tpListaObjs* )IupGetAttribute( IupCanvas, SZ_OBJETOS ) ;

   
/* corrige coordenada Y */
   iCoordY = canvasCorrectYAxis( IupCanvas, iCoordY ) ; 
      
   if ( iPress ) {   /* testa se pressiona boto */
         
      tpObjeto Objeto ; /* vai ser o objeto */
           
   /* testa se algum dos objetos foi selecionado */
      iAchou = iBuscarObj( MeusObjs, iCoordX, iCoordY ) ;
      
   /* desceleciona antigo objeto selecionado */
      Desselecionar( iAchou, MeusObjs, IupCanvas ) ;
      
   /* se no selecionou nenhum objeto, sai da funo */
      if ( iAchou < 0 ) {
         return IUP_DEFAULT ; /* Retornando para o "iup main loop" */
      } /* if */
         
   /* pega o objeto do ndice encontrado */
      Objeto = DarObjeto( MeusObjs, iAchou ) ;
      
      switch ( iBotao ) {

         case IUP_BUTTON1 :
         
         /* se o botao apertado foi o esquerdo, vai haver translao */
            
            cdForeground( CD_RED ) ; /* muda para cor vermelha */
         
         /* ao de movimento agora registrada para translao (arrasto) */
            IupSetFunction( "motion_act", iTransMoveCb ) ;
      
            break ;

         case IUP_BUTTON3 :

         /* se o botao apertado foi o direito, vai haver escala */
      
            cdForeground( CD_BLUE ) ; /* muda para cor azul */
         
         /* ao de movimento agora registrada para translao (arrasto) */
            IupSetFunction( "motion_act", iEscMoveCb ) ;
      
            break ;
            
         default : /* boto do meio desseleciona! */
            
            Desselecionar( -1, MeusObjs, IupCanvas ) ;
            return IUP_DEFAULT ;
         
      } /* switch */  

   /* desenha o objeto selecionado em outra cor */ 
      DesenharObjeto( IupCanvas, Objeto ) ;
        
      cdForeground( CD_BLACK ) ;  /* muda para cor preta */
      iGuardarPontoX( iCoordX ) ; /* guarda o X do mouse */
      iGuardarPontoY( iCoordY ) ; /* guarda o Y do mouse */
   
   } else {

      tpObjeto  Objeto   ; /* objeto selecionado */
      tpTipoObj Tipo     ; /* tipo do objeto */
      int       iCoordX1 , /* X1 do objeto */
                iCoordX2 , /* X2 do objeto */
                iCoordX3 , /* X3 do objeto */
                iCoordY1 , /* Y1 do objeto */
                iCoordY2 , /* Y2 do objeto */
                iCoordY3 , /* Y3 do objeto */
                iIndice  ; /* o ndice do objeto */

   /* se no selecionou nenhum objeto, sai da funo */
      if ( iAchou < 0 ) {
         return IUP_DEFAULT ; /* Retornando para o "iup main loop" */
      } /* if */

      iIndice = iGuardarIdx( -1 ) ; /* altera dados do objeto */
         
      switch ( iBotao ) {

         case IUP_BUTTON1 : /* conclui translao */
            TransladarObj( iIndice, MeusObjs, &Objeto, &Tipo,
                           &iCoordX1, &iCoordX2, &iCoordX3, &iCoordY1,
                           &iCoordY2, &iCoordY3, iCoordX, iCoordY ) ;
            break ;

         case IUP_BUTTON3 : /* conclui escala */
            EscalarObj( iIndice, MeusObjs, &Objeto, &Tipo, &iCoordX1, &iCoordX2,
                                           &iCoordX3, &iCoordY1, &iCoordY2,
                                           &iCoordY3, iCoordX, iCoordY ) ;
            break ;

         default : /* boto do meio no faz nada! */

            return IUP_DEFAULT ;

      } /* switch */

   /* altera objeto */
      InserirObj( MeusObjs, iIndice, Tipo, iCoordX1, iCoordX2, iCoordX3,
                                           iCoordY1, iCoordY2, iCoordY3 ) ;

      objDraw( IupCanvas, MeusObjs ) ; /* redesenha o canvas */

   /* resseleciona o objeto */
      Desselecionar( iAchou, MeusObjs, IupCanvas ) ;

      Objeto = DarObjeto( MeusObjs, iIndice ) ; /* pega novo objeto */

   /* se o boto solto foi o esquerdo, objeto selecionado fica vermelho */
      if ( iBotao == IUP_BUTTON1 ) {
         cdForeground( CD_RED ) ;
      } /* if */

   /* se o boto solto foi o direito, objeto selecionado fica azul */
      if ( iBotao == IUP_BUTTON3 ) {
         cdForeground( CD_BLUE ) ;
      } /* if */

   /* desenha o objeto selecionado em outra cor */
      DesenharObjeto( IupCanvas, Objeto ) ;

      cdForeground( CD_BLACK ) ; /* muda para cor preta */

   /* ao de movimento volta a ser a padro */
      IupSetFunction( "motion_act", motion_cb ) ;

   } /* if */

   return IUP_DEFAULT ; /* Retornando para o "iup main loop" */

} /* iSelBotaoCb */


/*==============================================================================
*Nome: iExcBotaoCb
*=============================================================================*/

int iExcBotaoCb( Ihandle* IupCanvas , 
                 int      iBotao    , 
                 int      iPress    , 
                 int      iCoordX   ,
                 int      iCoordY    ) {
   
   tpListaObjs* MeusObjs  ;  /* lista de objetos */

/* pega lista de objetos */
   MeusObjs = ( tpListaObjs* )IupGetAttribute( IupCanvas, SZ_OBJETOS ) ;

   if ( iBotao == IUP_BUTTON1 ) {
      
   /* corrige coordenada Y */
      iCoordY = canvasCorrectYAxis( IupCanvas, iCoordY ) ; 
      
      if ( iPress ) {   /* testa se pressiona boto */
         
      /* testa se algum dos objetos foi selecionado */
         int iAchou = iBuscarObj( MeusObjs, iCoordX, iCoordY ) ;  

      /* se no achou nenhum objeto, sai da funo */
         if ( iAchou < 0 ) {
            return IUP_DEFAULT ; /* Retornando para o "iup main loop" */
         } /* if */
         
         ExcluirObj( MeusObjs, iAchou ) ;  /* exclui o objeto achado */
         objDraw( IupCanvas, MeusObjs ) ;  /* redesenha os outros objetos */
      
      } /* if */
  
   } /* if */

   return IUP_DEFAULT ; /* retornando para o "iup main loop" */

} /* iExcBotaoCb */


/*==============================================================================
*Nome: iRetBotaoCb
*=============================================================================*/

int iRetBotaoCb( Ihandle* IupCanvas , 
                 int      iBotao    , 
                 int      iPress    , 
                 int      iCoordX   ,
                 int      iCoordY    ) {
  
   tpListaObjs* MeusObjs ; /* lista de objetos */
   
/* pega lista de objetos */
   MeusObjs = ( tpListaObjs* )IupGetAttribute( IupCanvas, SZ_OBJETOS ) ;

/* s desenha retngulo se o boto pressionado for o esquerdo */
   if ( iBotao == IUP_BUTTON1 ) {
      
   /* corrige coordenada Y */
      iCoordY = canvasCorrectYAxis( IupCanvas, iCoordY ) ; 

      if ( iPress ) {   /* testa se pressiona boto */

      /* pressionou o boto, ento seta pontos iniciais do retngulo */
         iGuardarPontoX( iCoordX ) ;
         iGuardarPontoY( iCoordY ) ;
      
      /* ao de movimento agora registrada para desenhar retngulo */
         IupSetFunction( "motion_act", iRetMoveCb ) ; 

      } else {  /* soltou o boto */
      
      /* pega ponto inicial */
         int iCoordX0 = iGuardarPontoX( -1 ) ,
             iCoordY0 = iGuardarPontoY( -1 ) ;
       
      /* desenha retngulo em definitivo */ 
         canvasDrawRect( IupCanvas, iCoordX0, iCoordY0, iCoordX, iCoordY ) ;
         
      /* ao de movimento volta a ser a padro */
         IupSetFunction( "motion_act", motion_cb ) ;

      /* insere retngulo na lista */
         objInsertRect( MeusObjs, iCoordX0, iCoordY0, iCoordX, iCoordY ) ; 
                 
      } /* if */

   } /* if */
   
   return IUP_DEFAULT ; /* retornando para o "iup main loop" */

} /* iRetBotaoCb */


/*==============================================================================
*Nome: iEliBotaoCb
*=============================================================================*/

int iEliBotaoCb( Ihandle* IupCanvas , 
                 int      iBotao    , 
                 int      iPress    , 
                 int      iCoordX   ,
                 int      iCoordY    ) {

   tpListaObjs* MeusObjs ; /* lista de objetos */
   
/* pega lista de objetos */
   MeusObjs = ( tpListaObjs* )IupGetAttribute( IupCanvas, SZ_OBJETOS ) ;

/* s desenha elipse se o boto pressionado for o esquerdo */
   if ( iBotao == IUP_BUTTON1 ) {
      
   /* corrige coordenada Y */
      iCoordY = canvasCorrectYAxis( IupCanvas, iCoordY ) ; 

      if ( iPress ) {   /* testa se pressiona boto */

      /* pressionou o boto, ento seta pontos iniciais da elipse */
         iGuardarPontoX( iCoordX ) ;
         iGuardarPontoY( iCoordY ) ;
      
      /* ao de movimento agora registrada para desenhar elipse */
         IupSetFunction( "motion_act", iEliMoveCb ) ; 

      } else {  /* soltou o boto */
      
      /* pega ponto inicial */
         int iCoordX0 = iGuardarPontoX( -1 ) ,
             iCoordY0 = iGuardarPontoY( -1 ) ;
       
      /* desenha elipse em definitivo */ 
         canvasDrawEllipse( IupCanvas, iCoordX0, iCoordY0, iCoordX, iCoordY ) ;
        
      /* ao de movimento volta a ser a padro */
         IupSetFunction( "motion_act", motion_cb ) ;

      /* insere elipse na lista de objetos */
         objInsertEllipse( MeusObjs, iCoordX0, iCoordY0, iCoordX, iCoordY ) ;

      } /* if */

   } /* if */

   return IUP_DEFAULT ; /* Retornando para o "iup main loop" */

} /* iEliBotaoCb */


/*==============================================================================
*Nome: iTriBotaoCb
*=============================================================================*/

int iTriBotaoCb( Ihandle* IupCanvas , 
                 int      iBotao    , 
                 int      iPress    , 
                 int      iCoordX   ,
                 int      iCoordY    ) {

   static int i          = 0 ,  /* contador de retas */
              iCoorX[ 3 ]    ,  /* coordenadas x */
              iCoorY[ 3 ]    ;  /* coordenadas y */
             
   tpListaObjs* MeusObjs ; /* lista de objetos */
   
/* pega lista de objetos */
   MeusObjs = ( tpListaObjs* )IupGetAttribute( IupCanvas, SZ_OBJETOS ) ;

/* loop para as trs retas do tringulo */ 
   while ( i < 3 ) {

   /* s aceita se o boto pressionado for o esquerdo */
      if ( iBotao == IUP_BUTTON1 ) {

      /* ao de movimento agora registrada para desenhar tringulo */
         IupSetFunction( "motion_act", iTriMoveCb ) ;

         if ( iPress ) {   /* testa se pressiona boto */

         /* seta coordenadas do tringulo, corrigindo a Y */
            iCoorX[ i ] = iCoordX ;
            iCoorY[ i ] = canvasCorrectYAxis( IupCanvas, iCoordY ) ; 
            
            return IUP_DEFAULT ; /* Retornando para o "iup main loop" */

         } else {  /* soltou o boto */
            
         /* guarda ponto para a linha temporria */
            iGuardarPontoX( iCoorX[ i ] ) ;
            iGuardarPontoY( iCoorY[ i ] ) ;
      
            if ( i > 0 ) { 
      
            /* desenha reta i do tringulo em definitivo */ 
               canvasDrawLine( IupCanvas, iCoorX[ i - 1 ], iCoorY[ i - 1 ], 
                                          iCoorX[ i ], iCoorY[ i ] ) ;
            } /* if */
        
            i++ ; /* incrementa ponto do tringulo */
            
         /* s retorna para o "iup main loop" se i < 3 */
            if ( i < 3 ) {
               return IUP_DEFAULT ; /* Retornando para o "iup main loop" */
            } /* if */   

         } /* if */

      } else { /* se o boto apertado foi o esquerdo, sai simplesmente */
         return IUP_DEFAULT ; /* Retornando para o "iup main loop" */
      } /* if */

   } /* while */

/* ao de movimento volta a ser a padro */
   IupSetFunction( "motion_act", motion_cb ) ;
   
   i = 0 ; /* para ser possvel fazer outros tringulos */

/* fecha a ltima aresta do tringulo */
   canvasDrawLine( IupCanvas, iCoorX[ 2 ], iCoorY[ 2 ], 
                              iCoorX[ 0 ], iCoorY[ 0 ] ) ;

/* insere tringulo na lista de objetos */
   objInsertTriang( MeusObjs, iCoorX[ 0 ], iCoorY[ 0 ], iCoorX[ 1 ],
                              iCoorY[ 1 ], iCoorX[ 2 ], iCoorY[ 2 ] ) ;

   return IUP_DEFAULT ; /* Retornando para o "iup main loop" */

} /* iTriBotaoCb */


/*==============================================================================
*Nome: iTransMoveCb
*=============================================================================*/

int iTransMoveCb( Ihandle* IupCanvas ,
                  int      iCoordX   ,
                  int      iCoordY    ) {

   tpListaObjs* MeusObjs ; /* lista de objetos */
   tpObjeto     Objeto   ; /* objeto selecionado */
   tpTipoObj    Tipo     ; /* tipo do objeto */
   int          iCoordX1 , /* X1 do objeto */
                iCoordX2 , /* X2 do objeto */
                iCoordX3 , /* X3 do objeto */
                iCoordY1 , /* Y1 do objeto */
                iCoordY2 , /* Y2 do objeto */
                iCoordY3 , /* Y3 do objeto */
                iIndice  ; /* o ndice do objeto */

/* pega lista de objetos */
   MeusObjs = ( tpListaObjs* )IupGetAttribute( IupCanvas, SZ_OBJETOS ) ;

/* corrige coordenada Y */
   iCoordY = canvasCorrectYAxis( IupCanvas, iCoordY ) ;

   iIndice = iGuardarIdx( -1 ) ; /* pega ndice do objeto */

/* translada objeto */
   TransladarObj( iIndice, MeusObjs, &Objeto, &Tipo, &iCoordX1, &iCoordX2,
                                          &iCoordX3, &iCoordY1, &iCoordY2, 
                                          &iCoordY3, iCoordX, iCoordY ) ;
/* desenhar objeto temporrio */
   DesenharObjTemp( IupCanvas, Tipo, iCoordX1, iCoordX2, iCoordX3, 
                                     iCoordY1, iCoordY2, iCoordY3 ) ;

   return IUP_DEFAULT ;  /* Retornando para o "iup main loop" */

} /* iTransMoveCb */


/*==============================================================================
*Nome: iEscMoveCb
*=============================================================================*/

int iEscMoveCb( Ihandle* IupCanvas , 
                int      iCoordX   , 
                int      iCoordY    ) {

   tpListaObjs* MeusObjs ; /* lista de objetos */
   tpObjeto     Objeto   ; /* objeto selecionado */
   tpTipoObj    Tipo     ; /* tipo do objeto */
   int          iCoordX1 , /* X1 do objeto */
                iCoordX2 , /* X2 do objeto */
                iCoordX3 , /* X3 do objeto */
                iCoordY1 , /* Y1 do objeto */
                iCoordY2 , /* Y2 do objeto */
                iCoordY3 , /* Y3 do objeto */
                iIndice  ; /* o ndice do objeto */

/* pega lista de objetos */
   MeusObjs = ( tpListaObjs* )IupGetAttribute( IupCanvas, SZ_OBJETOS ) ;

/* corrige coordenada Y */
   iCoordY = canvasCorrectYAxis( IupCanvas, iCoordY ) ;

   iIndice = iGuardarIdx( -1 ) ; /* pega ndice do objeto */

/* escala objeto */
   EscalarObj( iIndice, MeusObjs, &Objeto, &Tipo, &iCoordX1, &iCoordX2,
               &iCoordX3, &iCoordY1, &iCoordY2, &iCoordY3, iCoordX, iCoordY ) ;

/* desenhar objeto temporrio */
   DesenharObjTemp( IupCanvas, Tipo, iCoordX1, iCoordX2, iCoordX3, 
                                     iCoordY1, iCoordY2, iCoordY3 ) ;

   return IUP_DEFAULT ;  /* Retornando para o "iup main loop" */

} /* iEscMoveCb */


/*==============================================================================
*Nome: iRetMoveCb
*=============================================================================*/

int iRetMoveCb( Ihandle* IupCanvas , 
                int      iCoordX   , 
                int      iCoordY    ) {

/* pega coordenadas iniciais */
   int iCoordX0 = iGuardarPontoX( -1 ) ,
       iCoordY0 = iGuardarPontoY( -1 ) ;
   
/* corrige coordenada Y */
   iCoordY = canvasCorrectYAxis( IupCanvas, iCoordY ) ; 
   
/* desenha retngulo temporrio */
   canvasDrawTempRect( IupCanvas, iCoordX0, iCoordY0, iCoordX, iCoordY ) ; 

   return IUP_DEFAULT ; /* Retornando para o "iup main loop" */

} /* iRetBotaoCb */


/*==============================================================================
*Nome: iEliMoveCb
*=============================================================================*/

int iEliMoveCb( Ihandle* IupCanvas , 
                int      iCoordX   , 
                int      iCoordY    ) {

/* pega ponto inicial */
   int iCoordX0 = iGuardarPontoX( -1 ) ,
       iCoordY0 = iGuardarPontoY( -1 ) ;
   
/* corrige coordenada Y */
   iCoordY = canvasCorrectYAxis( IupCanvas, iCoordY ) ; 
   
/* desenha elipse temporrio */
   canvasDrawTempEllipse( IupCanvas, iCoordX0, iCoordY0, iCoordX, iCoordY ) ; 

   return IUP_DEFAULT ; /* Retornando para o "iup main loop" */

} /* iEliBotaoCb */


/*==============================================================================
*Nome: iTriMoveCb
*=============================================================================*/

int iTriMoveCb( Ihandle* IupCanvas , 
                int      iCoordX   , 
                int      iCoordY    ) {

/* pega ponto inicial */
   int iCoordX0 = iGuardarPontoX( -1 ) ,
       iCoordY0 = iGuardarPontoY( -1 ) ;
   
/* corrige coordenada Y */
   iCoordY = canvasCorrectYAxis( IupCanvas, iCoordY ) ; 
   
/* desenha reta temporria */
   canvasDrawTempLine( IupCanvas, iCoordX0, iCoordY0, iCoordX, iCoordY ) ; 

   return IUP_DEFAULT ; /* Retornando para o "iup main loop" */

} /* iTriBotaoCb */


/* callbacks.c */