/** MODIFICADO POR JULIANA **/

/*
%M Gerenciador de desenho no canvas iup
%a Marcelo Gattass
%d 28 mai 2000
%p Editor de Objetos (T3 do curso CGI2000)
*/
#include <stdlib.h>


 /*- Inclusao das bibliotecas IUP e CD: ------------------------------------*/
#include <iup.h>                /* prototypes do IUP */
#include <cd.h>                 /* prototypes do CD */
#include <cdiup.h>              /* driver do canvas do IUP */

typedef struct _2points {
   int flag;  /* = 0 pontos nao definidos */
   int x0,y0; /* first point */
   int x1,y1; /* second point */
} Tpoints;

 /* Contexto deste canvas (permite aplicao com vrios canvas):-----------*/
typedef struct _canvas_context {
    cdCanvas *cd_canvas;
    Tpoints  pts;
} CanvasContext;

/*
%F Cria um contexto para desenhar neste canvas 
*/
void canvasCreate(Ihandle *iup_canvas) 
{
    CanvasContext *cc = (CanvasContext *) malloc(sizeof(CanvasContext));
    cc->cd_canvas = cdCreateCanvas(CD_IUP, iup_canvas );
    cc->pts.flag  = 0;
    IupSetAttribute(iup_canvas,"CanvasContext",(Ihandle *)cc);
    return;
}

/*
%F Libera o contexto deste canvas 
*/
void canvasDestroy(Ihandle *iup_canvas) 
{
    CanvasContext *cc = (CanvasContext *) IupGetAttribute(iup_canvas,"CanvasContext");
    if (cc!=NULL) free(cc);
    return;
}


/*
%F Troca do sistema de eixos da IUP para o do CD
*/
int canvasCorrectYAxis(Ihandle *iup_canvas, int y) 
{
    CanvasContext *cc = (CanvasContext *) IupGetAttribute(iup_canvas,"CanvasContext");
    if (cc==NULL) return y;
    cdActivate(cc->cd_canvas); 
    cdUpdateYAxis(&y);
    return y;
}

/* 
%F Limpa a superficie de visualizacao deste canvas 
*/
void canvasClear(Ihandle *iup_canvas) 
{
    CanvasContext *cc = (CanvasContext *) IupGetAttribute(iup_canvas,"CanvasContext");
    if (cc==NULL) return;
    cdActivate(cc->cd_canvas); 
    cdClear();
}


/*********************LINE***********************************************************/

/* 
%F Desenha uma linha temporaria (que vai ser apagada depois)
*/     
void canvasDrawTempLine(Ihandle *iup_canvas, int x0, int y0, int x1, int y1) 
{
    CanvasContext *cc = (CanvasContext *) IupGetAttribute(iup_canvas,"CanvasContext");
    if (cc==NULL) return;

    cdActivate(cc->cd_canvas);
    cdWriteMode(CD_NOT_XOR);

    /* apaga a linha anterior (se existir) */
    if (cc->pts.flag!=0) 
        cdLine(cc->pts.x0,cc->pts.y0,cc->pts.x1,cc->pts.y1);

    /* salva a linha para ser apagada */
    cc->pts.flag=1;
    cc->pts.x0=x0;
    cc->pts.y0=y0;
    cc->pts.x1=x1;
    cc->pts.y1=y1;

    /* desenha a nova linha */
    cdLine(cc->pts.x0,cc->pts.y0,cc->pts.x1,cc->pts.y1);

    return;
}

/* desenha uma linha definitiva */
void canvasDrawLine(Ihandle *iup_canvas, int x0, int y0, int x1, int y1) 
{
    CanvasContext *cc = (CanvasContext *) IupGetAttribute(iup_canvas,"CanvasContext");
    if (cc==NULL) return;

    cdActivate(cc->cd_canvas);
    cdWriteMode(CD_NOT_XOR);

    /* apaga a linha anterior (se existir) */
    if (cc->pts.flag!=0) 
        cdLine(cc->pts.x0,cc->pts.y0,cc->pts.x1,cc->pts.y1);

    /* desenha a nova linha */
    cdWriteMode(CD_REPLACE);
    cdLine(x0,y0,x1,y1);

    /* Limpa o buffer da linha a ser apagada */
    cc->pts.flag=0;
    return;
}

/***********************RECT********************************************************/

/*
%F Desenha um retangulo temporario (que vai ser apagado depois) 
*/     
void canvasDrawTempRect(Ihandle *iup_canvas, int x0, int y0, int x1, int y1) 
{
    CanvasContext *cc = (CanvasContext *) IupGetAttribute(iup_canvas,"CanvasContext");
    if (cc==NULL) return;

    cdActivate(cc->cd_canvas);
    cdWriteMode(CD_NOT_XOR);

    /* apaga o rectangulo anterior (se existir) */
    if (cc->pts.flag!=0) 
        cdRect(cc->pts.x0,cc->pts.x1,cc->pts.y0,cc->pts.y1);

    /* salva o retangulo para ser apagado, colocando o minimo em 0 e o maximo em 1 */
    cc->pts.x0=(x0<x1)?x0:x1;
    cc->pts.y0=(y0<y1)?y0:y1;
    cc->pts.x1=(x0>x1)?x0:x1;
    cc->pts.y1=(y0>y1)?y0:y1;
    
    cdRect(cc->pts.x0,cc->pts.x1,cc->pts.y0,cc->pts.y1);
    cc->pts.flag=1;

    return;
}

/*
%F Desenha um retangulo definitivo 
*/
void canvasDrawRect(Ihandle *iup_canvas, int x0, int y0, int x1, int y1) 
{
    CanvasContext *cc = (CanvasContext *) IupGetAttribute(iup_canvas,"CanvasContext");
    int xmin,xmax,ymin,ymax;
    if (cc==NULL) return;

    cdActivate(cc->cd_canvas);
    cdWriteMode(CD_NOT_XOR);

    /* apaga o retangulo anterior (se existir) */
    if (cc->pts.flag!=0) 
        cdRect(cc->pts.x0,cc->pts.x1,cc->pts.y0,cc->pts.y1);

    /* desenha o novo retangulo */
    cdWriteMode(CD_REPLACE);
    xmin=(x0<x1)?x0:x1;
    ymin=(y0<y1)?y0:y1;
    xmax=(x0>x1)?x0:x1;
    ymax=(y0>y1)?y0:y1;
    cdRect(xmin,xmax,ymin,ymax);

    /* Limpa o buffer do retangulo a ser apagado */
    cc->pts.flag=0;
    return;
}

/**************************ELLIPSE**************************************************/

/*
%F Desenha uma elipse temporaria (que vai ser apagada depois) 
*/     
void canvasDrawTempEllipse(Ihandle *iup_canvas, int x0, int y0, int x1, int y1) 
{
    CanvasContext *cc = (CanvasContext *) IupGetAttribute(iup_canvas,"CanvasContext");
    int xc = (x0+x1)/2;
    int yc = (y0+y1)/2;
    int w  = (x1>x0)? x1-x0 : x0-x1;
    int h  = (y1>y0)? y1-y0 : y0-y1;

    if (cc==NULL) return;

    cdActivate(cc->cd_canvas);
    cdWriteMode(CD_NOT_XOR);

    /* apaga a ellipse anterior (se existir) */
    if (cc->pts.flag!=0) 
        cdArc(cc->pts.x0,cc->pts.y0,cc->pts.x1,cc->pts.y1, 0.0, 360.0);
        
    /* salva o elipse para ser apagada, colocando o minimo em 0 e o maximo em 1 */
    cc->pts.x0=xc;
    cc->pts.y0=yc;
    cc->pts.x1=w;
    cc->pts.y1=h;
    cc->pts.flag=1;
    cdArc(xc, yc, w, h, 0.0, 360.0);

    return;
}

/*
%F Desenha um elipse definitivo 
*/
void canvasDrawEllipse(Ihandle *iup_canvas, int x0, int y0, int x1, int y1) 
{
    CanvasContext *cc = (CanvasContext *) IupGetAttribute(iup_canvas,"CanvasContext");
    int xc = (x0+x1)/2;
    int yc = (y0+y1)/2;
    int w  = (x1>x0)? x1-x0 : x0-x1;
    int h  = (y1>y0)? y1-y0 : y0-y1;

    if (cc==NULL) return;

    cdActivate(cc->cd_canvas);
    cdWriteMode(CD_NOT_XOR);

    /* apaga o elipse anterior (se existir) */
    if (cc->pts.flag!=0) 
        cdArc(cc->pts.x0,cc->pts.y0,cc->pts.x1,cc->pts.y1, 0.0, 360.0);

    /* desenha a nova elipse */
    cdWriteMode(CD_REPLACE);
    cdArc(xc, yc, w, h, 0.0, 360.0);

    /* Limpa o buffer da elipse a ser apagada */
    cc->pts.flag=0;
    return;
}
