/** MODIFICADO POR JULIANA **/

/* 
%M Modulo de gerencia dos objetos
*/

#ifndef OBJECTS_H
#define OBJECTS_H 1

#define I_MAX_OBJS 200  /* nmero mximo de objetos possvel */
#define SZ_MEM     "Nao tem memoria para criar lista de objetos!\n"


/*==============================================================================
*NOME:  tpTipoObj
*Descricao do tipo:
*   Enumerao dos identificadores de objeto.
*
*Valores possiveis para o tipo enumeracao:
*   ID_Retangulo - objeto do tipo retngulo;
*   ID_Elipse    - objeto do tipo elipse e
*   ID_Triangulo - objeto do tipo tringulo.
*=============================================================================*/

typedef enum {
   ID_Retangulo ,
   ID_Elipse    ,
   ID_Triangulo
} tpTipoObj ;


/*==============================================================================
*NOME:  tpObjeto
*Descricao do tipo:
*   Representa um objeto que est ou vai ser representado no canvas.
*
*Campos:
*   TipoObjeto - pode ser retngulo, elipse ou tringulo;
*   iCoordX1   - Coordenada X do ponto 1 do objeto;
*   iCoordX2   - Coordenada X do ponto 2 do objeto;
*   iCoordX3   - Coordenada X do ponto 3 do objeto (= -1 em elipse e retngulo);
*   iCoordY1   - Coordenada Y do ponto 1 do objeto;
*   iCoordY2   - Coordenada Y do ponto 2 do objeto e
*   iCoordY3   - Coordenada Y do ponto 3 do objeto (= -1 em elipse e retngulo).
*=============================================================================*/

typedef struct {
   tpTipoObj TipoObjeto ;
   int       iCoordX1   ,
             iCoordX2   ,
             iCoordX3   ,
             iCoordY1   ,
             iCoordY2   ,
             iCoordY3   ;
} tpObjeto ;


/*==============================================================================
*NOME:  tpListaObjs
*Descricao do tipo:
*   Contm uma lista de objetos e o tamanho da lista.
*
*Campos:
*   Objetos  -  um vetor de objetos de no mximo I_MAX_OBJS e
*   iNumObjs - o nmero de objetos na lista.
*=============================================================================*/

typedef struct {
   tpObjeto Objetos[ I_MAX_OBJS ] ;
   int      iNumObjs              ;
} tpListaObjs ;


int objRead( char*        filename ,
             tpListaObjs* MeusObjs  ) ;

int objSave( char*        filename ,
             tpListaObjs* MeusObjs  ) ;

int objInsertRect( tpListaObjs* MeusObjs ,
                   int          x0       , 
                   int          y0       ,  
                   int          x1       , 
                   int          y1        ) ;

int objInsertEllipse( tpListaObjs* MeusObjs ,
                      int x0                , 
                      int y0                , 
                      int x1                , 
                      int y1                 ) ;

int objInsertTriang( tpListaObjs* MeusObjs ,
                     int x0                , 
                     int y0                , 
                     int x1                , 
                     int y1                , 
                     int x2                , 
                     int y2                 ) ;


/*==============================================================================
*Nome: Inicializar
*Descrio da Funo: funo para inicializar a lista de objetos.
*
*Valor retornado
*   a lista de objetos.
*=============================================================================*/

tpListaObjs* Inicializar( void ) ;


/*==============================================================================
*Nome: ZerarObjetos
*Descrio da Funo: funo para zerar o nmero de objetos da lista.
*
*Parmetro recebido
*   MeusObjs - a lista de objetos.
*
*Parmetro retornado:
*   MeusObjs - a lista de objetos com zero elementos.
*=============================================================================*/

void ZerarObjetos( tpListaObjs* MeusObjs ) ;


/*==============================================================================
*Nome: iDarCoordX1
*Descrio da Funo: retorna a coordenada X1 de um objeto.
*
*Valor retornado
*   a coordenada X1 do objeto.
*=============================================================================*/

int iDarCoordX1( tpObjeto Objeto ) ;


/*==============================================================================
*Nome: iDarCoordX2
*Descrio da Funo: retorna a coordenada X2 de um objeto.
*
*Valor retornado
*   a coordenada X2 do objeto.
*=============================================================================*/

int iDarCoordX2( tpObjeto Objeto ) ;


/*==============================================================================
*Nome: iDarCoordX3
*Descrio da Funo: retorna a coordenada X3 de um objeto.
*
*Valor retornado
*   a coordenada X3 do objeto.
*=============================================================================*/

int iDarCoordX3( tpObjeto Objeto ) ;


/*==============================================================================
*Nome: iDarCoordY1
*Descrio da Funo: retorna a coordenada Y1 de um objeto.
*
*Valor retornado
*   a coordenada Y1 do objeto.
*=============================================================================*/

int iDarCoordY1( tpObjeto Objeto ) ;


/*==============================================================================
*Nome: iDarCoordY2
*Descrio da Funo: retorna a coordenada Y2 de um objeto.
*
*Valor retornado
*   a coordenada Y2 do objeto.
*=============================================================================*/

int iDarCoordY2( tpObjeto Objeto ) ; 


/*==============================================================================
*Nome: iDarCoordY3
*Descrio da Funo: retorna a coordenada Y3 de um objeto.
*
*Valor retornado
*   a coordenada Y3 do objeto.
*=============================================================================*/

int iDarCoordY3( tpObjeto Objeto ) ;
 

/*==============================================================================
*Nome: DarTipo
*Descrio da Funo: retorna o tipo de um objeto.
*
*Valor retornado
*   o tipo do objeto.
*=============================================================================*/

int DarTipo( tpObjeto Objeto ) ;
 

/*==============================================================================
*Nome: DarObjeto
*Descrio da Funo: funo que devolve um objeto dado o seu ndice na lista.
*
*Parmetros recebidos
*   MeusObjs - a lista de objetos e
*   iIndice  - ndice do objeto procurado.
*
*Valor retornado
*   o objeto do ndice dado como parmetro.
*=============================================================================*/

tpObjeto DarObjeto( tpListaObjs* MeusObjs ,
                    int          iIndice   ) ;


/*==============================================================================
*Nome: iDarNumObjs
*Descrio da Funo: funo que devolve o nmero de objetos na lista.
*
*Parmetro recebido
*   MeusObjs - a lista de objetos.
*
*Valor retornado
*   o nmero de objetos na lista.
*=============================================================================*/

int iDarNumObjs( tpListaObjs* MeusObjs ) ;


/*==============================================================================
*Nome: iBuscarObj
*Descrio da Funo: funo para buscar um objeto na lista de objetos.
*
*Parmetros recebidos
*   MeusObjs - a lista de objetos;
*   iCoordX  - a coordenada X de onde est o mouse e
*   iCoordY  - a coordenada Y de onde est o mouse.
*
*Valor retornado
*   -> o ndice do objeto se o objeto est na lista ou
*   -> -1 se o objeto no est na lista
*=============================================================================*/

int iBuscarObj( tpListaObjs* MeusObjs ,
                int          iCoordX  ,
                int          iCoordY   ) ;


/*==============================================================================
*Nome: InserirObj
*Descrio da Funo: funo para inserir um objeto na lista de objetos.
*
*Parmetros recebidos
*   MeusObjs   - a lista de objetos ;
*   iIndice    - o ndice do objeto a ser inserido;
*   TipoObjeto - pode ser retngulo, elipse ou tringulo;
*   iCoordX1   - Coordenada X do ponto 1 do objeto;
*   iCoordX2   - Coordenada X do ponto 2 do objeto;
*   iCoordX3   - Coordenada X do ponto 3 do objeto (= -1 em elipse e retngulo);
*   iCoordY1   - Coordenada Y do ponto 1 do objeto;
*   iCoordY2   - Coordenada Y do ponto 2 do objeto e
*   iCoordY3   - Coordenada Y do ponto 3 do objeto (= -1 em elipse e retngulo).
*
*Parmetro retornado:
*   MeusObjs - a lista de objetos com mais um elemento.
*=============================================================================*/

void InserirObj( tpListaObjs* MeusObjs   ,
                 int          iIndice    ,
                 tpTipoObj    TipoObjeto ,
                 int          iCoordX1   ,
                 int          iCoordX2   ,
                 int          iCoordX3   ,
                 int          iCoordY1   ,
                 int          iCoordY2   ,
                 int          iCoordY3    ) ;


/*==============================================================================
*Nome: ExcluirObj
*Descrio da Funo: funo para excluir um objeto da lista de objetos.
*
*Parmetros recebidos
*   MeusObjs - a lista de objetos e
*   iIndice  - o ndice do objeto a ser excludo. 
*
*Parmetro retornado:
*   MeusObjs - a lista de objetos com menos um elemento.
*=============================================================================*/

void ExcluirObj( tpListaObjs* MeusObjs , 
                 int          iIndice   ) ;


#endif